//
// XMM Science Analysis System
//
// Giuseppe Vacanti (cosine) 
// June 2000
//
#include "TimeOrderedTable.h"
#include <errstr.h>
#include <Tracer.h>
#include <SasDalUtils.h>
#include <utility>
#include <algorithm>

static bool comPair(const pair<STime, unsigned>& rhs, const pair<STime, unsigned>& lhs)
{
  PROC
  RETURN rhs.first < lhs.first;
}
  

TimeOrderedTable::TimeOrderedTable(const Table *tab, const string& timename = "TIME")
  : _tab(tab),
    _stimes(tab->rows()),
    _col(tab->column(timename)->data())
    
{
  PROC
  using namespace SasDalUtils;
  for(unsigned i = 0; i < _tab->rows(); i++){
    _stimes[i] = pair<STime, unsigned>(STime(getStringColumnRow(_col, i)),i);
  }
  // The table must be sorted in time ascending order.
  pair<STime, unsigned> prev = _stimes[0];
  for(unsigned i = 1; i < _stimes.size(); i++){
    if(prev.first > _stimes[i].first)
      errstr << tab->name() + " is not sorted in time ascending order at row " << i << Sas::error("NotATimeSortedTable");
    prev = _stimes[i];
  }  
  ENDPROC
}

TimeOrderedTable::~TimeOrderedTable () { PROC delete _col; ENDPROC }

const Table& TimeOrderedTable::table() const
{
  PROC
  RETURN *_tab;
}

unsigned TimeOrderedTable::rows() const
{
  PROC
  RETURN _tab->rows();
}

const STime& TimeOrderedTable::first_time() const
{
  PROC
  RETURN _stimes[0].first;
}

const STime& TimeOrderedTable::last_time() const
{
  PROC
  RETURN _stimes[rows() - 1].first;
}

unsigned TimeOrderedTable::lower_bound(const STime& t) const
{
  PROC
  CI b = ::lower_bound(_stimes.begin(), _stimes.end(),
		       make_pair(t,0), comPair);
  if( b == _stimes.end() ||
      b == _stimes.begin()) {
    errstr << "Looking for the first row such that " << t << " <= time[row] failed. Available time range: " << first_time() << " - " << last_time() << Sas::message(Msg::AppLib, Msg::Verbose);
    ALTRETURNV(rows())
  }
  RETURN b->second;
}

unsigned TimeOrderedTable::upper_bound(const STime& t) const
{
  PROC
  CI b = ::upper_bound(_stimes.begin(), _stimes.end(),
		       make_pair(t,0), comPair);
  if(b == _stimes.end() ||
     b == _stimes.begin()) {
    errstr << "Looking for tha last row such that " << t << " >= time[row] failed. Available time range: " << first_time() << " - " << last_time() << Sas::message(Msg::AppLib, Msg::Verbose);
    ALTRETURNV(rows())
  }
  RETURN (b - 1)->second;
}

//  unsigned TimeOrderedTable::lower_bound(const STime& t) const
//  {
//    PROC
//    CI b = ::lower_bound(_stimes.begin(), _stimes.end(),
//  		       make_pair(t,0), comPair);
//    //		       pair<STime, unsigned>(t,0), comPair);
//    if(b == _stimes.end() ||
//       t < _stimes.begin()->first){ // this also covers the case when b = _stimes.begin()
//      errstr << "Looking for the first row such that " << t << " <= time[row] failed. Available time range: " << first_time() << " - " << last_time() << Sas::message(Msg::AppLib, Msg::Verbose);
//      ALTRETURNV(rows())
//    }
//    if(b == _stimes.begin())
//      ALTRETURNV(b->second)
//    else
//      RETURN (b - 1)->second;
//  }

//  unsigned TimeOrderedTable::upper_bound(const STime& t) const
//  {
//    PROC
//    CI b = ::upper_bound(_stimes.begin(), _stimes.end(),
//  		       make_pair(t,0), comPair);
//    //		       pair<STime, unsigned>(t,0), comPair);
//    if(b == _stimes.end()){
//      errstr << "Looking for tha last row such that " << t << " >= time[row] failed. Available time range: " << first_time() << " - " << last_time() << Sas::message(Msg::AppLib, Msg::Verbose);
//      ALTRETURNV(rows())
//    }
//    if(b == _stimes.end() - 1)
//      ALTRETURNV(b->second)
//    else
//      RETURN b->second;
//  }

unsigned TimeOrderedTable::last_less_than(const STime& t) const
{
  PROC
  for(unsigned i = rows() - 1; i != 0; --i)
    if(time_of_row(i) <= t)
      ALTRETURNV(i)
  RETURN rows();
}

unsigned TimeOrderedTable::first_less_than(const STime& t) const
{
  PROC
  for(unsigned i = 0; i < rows(); ++i)
    if(time_of_row(i) <= t)
      ALTRETURNV(i)
  RETURN rows();
}

const STime& TimeOrderedTable::time_of_row(const unsigned row) const
{
  PROC
  RETURN _stimes[row].first;
}

