//
// XMM Science Analysis System
//
// Giuseppe Vacanti (cosine) 
// June 2000
//
#ifndef TIMEORDEREDTABLE_H
#define TIMEORDEREDTABLE_H

#include <Dal.h>
#include <STime.h>
#include <string>
#include <vector>
#include <utility>

// A DAL table sorted in time ascending order.
//
// lower_bound returns the first row where t >= time(row), or
// tab->rows() if t is outside of the time range of the table.
//
// upper_bound returns the last row where t <= time(row), or
// tab->rows() if t is outside of the time range of the table.

class TimeOrderedTable {
 public:
  TimeOrderedTable(const Table *tab, const string& timename = "TIME");
  ~TimeOrderedTable();
  const Table& table() const;
  unsigned rows() const;
  unsigned lower_bound(const STime& t) const;
  unsigned upper_bound(const STime& t) const;
  unsigned last_less_than(const STime& t) const;
  unsigned first_less_than(const STime& t) const;
  const STime& time_of_row(const unsigned row) const;
  const STime& first_time() const;
  const STime& last_time() const;

 private:

  const Table *_tab;
  vector<pair<STime, unsigned> > _stimes;
  ColumnData *_col;

  typedef vector<pair<STime, unsigned> >::const_iterator CI;

};

#endif
