//
// XMM Science Analysis System
//
// Giuseppe Vacanti (cosine) 
// September 2000
//

// calmodes /path/to/odf/directory
//
// It does not make use of the OAL.
//
// Examine all the *.FIT files in an ODF directory, and for those
// corresponding to a science (data) mode try to determine the science
// (camera) mode.
//
// Must have SAS_CCF set up correctly.
//

#include <CalServer.h>
#include <Odf.h>
#include <string>
#include <vector>
#include <algorithm>

#include <iostream>
#include <Directory.h>

int main(int argc, char *argv[]) {

  if(argc == 1){
    cerr << "Usage: " << argv[0] << " <ODF dir> <fileglob>" << endl;
    exit(argc);
  }

  string fn(argv[1]);

  cout << "Directory: " << fn << endl;
  cout << "Filter   : " << argv[2] << endl;

  vector<Xmm::DataMode> science_modes;
  science_modes.push_back(Xmm::IMAGING);
  science_modes.push_back(Xmm::TIMING);
  science_modes.push_back(Xmm::REDUCED_IMAGING);
  science_modes.push_back(Xmm::COMPRESSED_TIMING);
  science_modes.push_back(Xmm::BURST);
  science_modes.push_back(Xmm::DATA_SPECTROSCOPY);
  science_modes.push_back(Xmm::HIGH_TIME_RESOLUTION);
  science_modes.push_back(Xmm::DATA_FAST);
  science_modes.push_back(Xmm::DATA_DIAGNOSTIC);
  science_modes.push_back(Xmm::DISCARDED_LINES_DATA);
  science_modes.push_back(Xmm::PRIORITY_FIELD_ACQUISITION);
  
  Directory d(fn);
  d.filter(argv[2]);

  for(Directory::iterator k = d.begin(); k != d.end(); ++k){
    OdfSet s((*k)->basename(), fn);
    cout << "Considering " << (*k)->basename() << " whose data mode is "
	 << s.dataMode() <<endl;
    vector<Xmm::DataMode>::const_iterator p = find(science_modes.begin(),
						   science_modes.end(),
						   s.dataMode());
    if(p != science_modes.end()){
      calServer.setState(s.name(), true, true);
      CalState *cs = calServer.state();
    //    cout << *cs << endl;
      ChoiceStateVariable<Mode> *m = cs->mode();
      if(m->isUncertain()){
	cerr << "* The cal cannot uniquely determined the calibration mode."
	     << endl;
      }
      cout << s.basename() << " has the following calibration mode " << Xmm::toString(m->get()) << endl;
    } 
  }
}
